/*******************************************************************************
 * Copyright (c) 2006 Phil Muldoon <pkmuldoon@picobot.org>.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Phil Muldoon <pmuldoon@redhat.com> - initial API and implementation
 *    Kyu Lee <klee@redhat.com>          - new execute method
 *******************************************************************************/
package org.eclipse.linuxtools.changelog.core.actions;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.IHandlerListener;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.linuxtools.changelog.core.ChangeLogWriter;
import org.eclipse.linuxtools.changelog.core.IParserChangeLogContrib;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;


/**
 * @author pmuldoon (Phil Muldoon)
 */
public class InsertChangeLogKeyHandler extends ChangeLogAction implements
		IHandler, IWorkbenchWindowActionDelegate {

	private IEditorPart currentEditor;

	public InsertChangeLogKeyHandler() {

		super();

	}

	String getEditorName() {
		if (currentEditor != null)
			return returnQualifedEditor(currentEditor.getClass());
		else
			return "";

	}

	String getEntryFilePath() {
		if (currentEditor != null)
			return getDocumentLocation(currentEditor, false);
		else
			return "";
	}

	IEditorPart getChangelog() {

		IConfigurationElement formatterConfigElement = extensionManager
				.getFormatterConfigElement();
		if (formatterConfigElement.getAttribute("inFile").toLowerCase().equals(
				"true")) {
			return currentEditor;
			// this formatter wants to use an external changelog file
		} else {
			IEditorPart changelog = null;

			IConfigurationElement nameElement = formatterConfigElement
					.getChildren()[0];
			if (nameElement.getAttribute("name") == null) {
				reportErr("Got non-name child with inFile set to False", null);
				return null;
			} else {
				pref_ChangeLogName = nameElement.getAttribute("name");
				changelog = getChangelog(getDocumentLocation(currentEditor,
						false));

				if (changelog == null) {
					changelog = askChangeLogLocation(getDocumentLocation(
							currentEditor, false));
				}

				return changelog;
			}
		}

	}

	String parseFunctionName(IParserChangeLogContrib parser) {

		try {
			return parser.parseCurrentFunction(currentEditor);
		} catch (CoreException e) {
			reportErr("Couldn't parse function name with "
					+ parser.getClass().toString(), null);
			return "";
		}

	}

	public Object execute(ExecutionEvent event) throws ExecutionException {

		try {
			currentEditor = getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().getActiveEditor();
		} catch (Exception e) {
			// no editor is active now so do nothing
			return null;
		}

		// make sure an editor is selected.
		if (currentEditor == null) {
			return null;
		}

		ChangeLogWriter clw = new ChangeLogWriter();

		// load settings from extensions + user pref.
		loadPreferences();

		// get file path from target file
		clw.setEntryFilePath(getEntryFilePath());

		// err check. do nothing if no file is being open/edited
		if (clw.getEntryFilePath() == "") {
			return null;
		}

		String editorName = getEditorName();

		// get a parser for this file
		IParserChangeLogContrib parser = extensionManager
				.getParserContributor(editorName);

		// if no parser for this type of document, then don't guess function
		// name
		// and set it as "".
		if (parser == null) {
			clw.setGuessedFName("");
		} else {
			// guess function name
			clw.setGuessedFName(parseFunctionName(parser));
		}

		// get formatter

		clw.setFormatter(extensionManager.getFormatterContributor(clw
				.getEntryFilePath(), pref_Formatter));

		// select changelog
		clw.setChangelog(getChangelog());

		// write to changelog

		clw.setDateLine(clw.getFormatter().formatDateLine(pref_AuthorName,
				pref_AuthorEmail));

		clw.setChangelogLocation(getDocumentLocation(clw.getChangelog(), true));

		clw.writeChangeLog();

		return null;
	}

	public void addHandlerListener(IHandlerListener handlerListener) {

	}

	public boolean isEnabled() {

		return true;
	}

	public boolean isHandled() {

		return true;
	}

	public void removeHandlerListener(IHandlerListener handlerListener) {

	}

	public void dispose() {

	}

	public void init(IWorkbenchWindow window) {

	}

	public void run(IAction action) {

		try {
			execute(null);
		} catch (ExecutionException e) {

			reportErr("Executing insert changelog failed", e);
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {

	}

}
